package com.parking.HAPEstimation;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import com.parking.main.Experiment;

/**
 * @author leon stenneth, lstennethATgmail.com
 *
 */

//POJO + parking availability increase / decrease measurement
public class HAPConstruction {
	private HistoricalModel hap;
	private static double qt;
	private static double Qt;
	private static HashMap <Time,Integer> historical_mean_map = new HashMap <Time,Integer>();
	private static HashMap <Time,Integer> historical_variance_map = new HashMap <Time, Integer>();
	private HashMap <Time,Double> hap_mean_map;
	private HashMap <Time,Double> hap_variance_map;
	Experiment hapExperiment;

	public HAPConstruction (){
		//hapExperiment = new Experiment();
		qt=0;
		Qt=0;
		hap = new HistoricalModel();	
//		historical_mean_map = new HashMap <Time,Integer>();
		hap_mean_map = new HashMap <Time,Double>();
		hap_variance_map = new HashMap <Time,Double>();
//		historical_variance_map = new HashMap <Time,Integer>();
	}

	public HistoricalModel getHap() {
		return hap;
	}

	public double getMean(ArrayList <Double> data)
    {
        double sum = 0.0;
        for(double a : data)
            sum += a;
         
        
        return sum/data.size();
    }

    public double getVariance(ArrayList <Double> data)
    {
            double mean = getMean(data);
            double temp = 0;
            for(double a :data)
                temp += (mean-a)*(mean-a);
                
           
            return temp/data.size();
    }

	public void setHap(HistoricalModel hap) {
		this.hap = hap;
	}

	public static double getqt() {
		return qt;
	}
	public void setqt(double qt) {
		this.qt = qt;
	}
	public static double  getQt() {
		return Qt;
	}
	public void setQt(double qt) {
		Qt = qt;
	}

	public void decreaseParkingAvailability(){
		double availability=0.0;
		double fraction=0.0;

		if(this.getHap().getB()==0)
			fraction=0;
		else
			fraction =(1-this.getHap().getFp())/(this.getHap().getB()*(1-this.getHap().getFn()));

		//System.out.println("decrease by "+fraction);

		availability= this.getHap().getAi()-fraction;
		this.getHap().setAi(availability);
	}


	public void increaseParkingAvailability(){
		double availability=0.0;
		double fraction=0.0;

		if(this.getHap().getB()==0)
			fraction=0;
		else
			fraction =(1-this.getHap().getFp())/(this.getHap().getB()*(1-this.getHap().getFn()));

		//System.out.println("increase by "+fraction);
		availability= this.getHap().getAi()+fraction;
		this.getHap().setAi(availability);

	}

	@SuppressWarnings("deprecation")
	public void computeHistoricMeanAndVarianceUsingHAP(double penetration_ratio,double fp,double fn,int hour, int min,int permitted_period_count){
		
		if(hap_mean_map.containsKey(new Time (hour,min,0))){
			System.out.println("found in historical statistics memory map...");
			this.setqt(hap_mean_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			this.setQt(hap_variance_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			return;
		}
		
		else {
			
			String meanWithVariance =  new Experiment().runHAPFromPAE(penetration_ratio, fp, fn, hour, min,permitted_period_count);
			double qt = Double.parseDouble(meanWithVariance.split(":")[0]);
			double Qt = Double.parseDouble(meanWithVariance.split(":")[1]);
			System.out.println("qt from hap "+qt);
			System.out.println("Qt from hap "+Qt);
			
			//capping HAP
			if (qt<0){
				this.setqt(0);
			}
			else if (qt>12){ //N =12 lazy code
				this.setqt(12);
			}
			else {
				this.setqt(qt);
			}
			//this.setqt(12);
			this.setQt(Qt);
			hap_mean_map.put(new Time (hour,min,0),this.getqt());
			hap_variance_map.put(new Time (hour,min,0),this.getqt());
		}
		
		
	}
	
	
	@SuppressWarnings("deprecation")
	public void computeMeanAndVarianceFromHistoryForATimePeriod(){
		if(historical_mean_map.containsKey(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0))){
			//System.out.println("found in historical statistics memory map...");
			//this.setqt(historical_mean_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			this.setQt(historical_variance_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			return;
		}

		else {
			try {
				Class.forName("org.postgresql.Driver").newInstance();
				Connection db=DriverManager.getConnection("jdbc:postgresql://localhost:5432/postgis","postgres","Yhcaapa3");	
				Statement stmt=db.createStatement();
				
				
				//String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from sfpark where bfid = '"+this.hap.getStreet_block_id()+"' and timestamp <='"+this.hap.getEndTimeFrameForHistory()+"' AND timestamp >= '"+this.hap.getStartTimeFrameForHistory()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(minute from timestamp) >= "+this.hap.getHistory_start_minute()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+" AND extract(minute from timestamp) <"+this.hap.getHistory_end_minute()+"";
				String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from block612032 where bfid = '"+this.hap.getStreet_block_id()+"' and timestamp <='"+this.hap.getEndTimeFrameForHistory()+"' AND timestamp >= '"+this.hap.getStartTimeFrameForHistory()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+"";
				System.out.println(historyQuery);
				
				ResultSet rs = stmt.executeQuery(historyQuery);
				while(rs.next()){
					//this.setqt(rs.getInt(1)); //setting the historical mean
					this.setQt(rs.getInt(2)); //setting the historical variance

					//historical_mean_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0),rs.getInt(1));
					historical_variance_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0),rs.getInt(2));
				}
				rs.close();
				stmt.close();
				db.close();
			}
			catch (Exception e){
				e.printStackTrace();

			}
		}
	}
	public void computeMeanAndVariance(){
		if(historical_mean_map.containsKey(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0))){
			//System.out.println("found in historical statistics memory map...");
			this.setqt(historical_mean_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			this.setQt(historical_variance_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0)));
			return;
		}

		else {
			try {
				Class.forName("org.postgresql.Driver").newInstance();
				Connection db=DriverManager.getConnection("jdbc:postgresql://localhost:5432/postgis","postgres","Yhcaapa3");	
				Statement stmt=db.createStatement();
				
				
				String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from block612032 where bfid = '"+this.hap.getStreet_block_id()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(minute from timestamp) >= "+this.hap.getHistory_start_minute()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+" AND extract(minute from timestamp) <"+this.hap.getHistory_end_minute()+"";
				//String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from sfpark where bfid = '"+this.hap.getStreet_block_id()+"' and timestamp <='"+this.hap.getEndTimeFrameForHistory()+"' AND timestamp >= '"+this.hap.getStartTimeFrameForHistory()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+"";
				//System.out.println(historyQuery);
				
				ResultSet rs = stmt.executeQuery(historyQuery);
				while(rs.next()){
					this.setqt(rs.getInt(1)); //setting the historical mean
					this.setQt(rs.getInt(2)); //setting the historical variance

					historical_mean_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0),rs.getInt(1));
					historical_variance_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),0),rs.getInt(2));
				}
				rs.close();
				stmt.close();
				db.close();
			}
			catch (Exception e){
				e.printStackTrace();

			}
		}
	}

	/*public HashMap<String, Long[]> getMeanAndVarianceFromMemory (HashMap<Timestamp,Integer> changes, Calendar calStart,Calendar calEnd,String hourMin){
		HashMap<String, Long[]> meanVarMap = null;
		ArrayList <Double> sumList = new ArrayList <Double> ();
		ArrayList <Double> dayList = null ; 
		ArrayList <Timestamp>changesKeyListDaily = new ArrayList<Timestamp>();
		ArrayList <Integer> changesAvlListDaily = new ArrayList<Integer>();
		
		double avl=-1.0;
		
		while(calStart.before(calEnd)){	
			avl=-1.0;
			
			for(Timestamp t:changes.keySet()){	
				Date d =new Date(t.getTime());
				if(calStart.getTime().getDate()==d.getDate()&&calStart.getTime().getMonth()==d.getMonth()){
					changesKeyListDaily.add(t);
					changesAvlListDaily.add(changes.get(t));
				}
				
				if(t.after(new Timestamp(calStart.getTimeInMillis()))){ //if it is after it is not good
					break;
				}
			}
			//---------now process the daliy lists
			double hourMinDouble = this.convertTimeStringToDouble(hourMin);
			//ArrayList <Double> timeList =this.convertTimeMapofStringToDoubles(changesKeyListDaily);
			//Timestamp closestTime = 
			//avl = 
			//---------
			
			if(avl!=-1)
				dayList.add(avl);
			else
				System.err.println("ERRRRRRRRRRRRRRR");
			
			calStart.add(Calendar.DATE, 1);
		}	
		return meanVarMap;
	}*/
	public double convertTimeStringToDouble(String s){
		 return Double.parseDouble(s.split(":")[0])+(Double.parseDouble(s.split(":")[1])/60);
	}
	
	public ArrayList <Double> convertTimeMapofStringToDoubles(ArrayList <String> list){
		Boolean status=false;
		String string="";
		ArrayList <Double> doubles = new ArrayList <Double> ();
		for(int index=0;index<list.size();index++){
			string=list.get(index);
			doubles.add(this.convertTimeStringToDouble(string)) ;//status return Double.parseDouble(s.split(":")[0])+(Double.parseDouble(s.split(":")[1])/60);
		}
		return doubles;
	}
	public void computeMeanAndVariancePerSecond(){
		if(historical_mean_map.containsKey(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),this.hap.getHistory_start_second()))){
			//System.out.println("found in historical statistics memory map...");
			this.setqt(historical_mean_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),this.hap.getHistory_start_second())));
			this.setQt(historical_variance_map.get(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),this.hap.getHistory_start_second())));
			return;
		}

		else {
			try {
				Class.forName("org.postgresql.Driver").newInstance();
				Connection db=DriverManager.getConnection("jdbc:postgresql://localhost:5432/postgis","postgres","Yhcaapa3");	
				Statement stmt=db.createStatement();
				
				
				String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from block612032 where bfid = '"+this.hap.getStreet_block_id()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(minute from timestamp) >= "+this.hap.getHistory_start_minute()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+" AND extract(minute from timestamp) <"+this.hap.getHistory_end_minute()+"";
				//String historyQuery = "select AVG (available) as mean, VARIANCE (available) as variance from sfpark where bfid = '"+this.hap.getStreet_block_id()+"' and timestamp <='"+this.hap.getEndTimeFrameForHistory()+"' AND timestamp >= '"+this.hap.getStartTimeFrameForHistory()+"' AND extract(hour from timestamp) >= "+this.hap.getHistory_start_hour()+" AND extract(hour from timestamp) <="+this.hap.getHistory_end_hour()+"";
				//System.out.println(historyQuery);
				
				ResultSet rs = stmt.executeQuery(historyQuery);
				while(rs.next()){
					this.setqt(rs.getInt(1)); //setting the historical mean
					this.setQt(rs.getInt(2)); //setting the historical variance

					historical_mean_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),this.hap.getHistory_start_second()),rs.getInt(1));
					historical_variance_map.put(new Time (this.hap.getHistory_start_hour(),this.hap.getHistory_start_minute(),this.hap.getHistory_start_second()),rs.getInt(2));
				}
				rs.close();
				stmt.close();
				db.close();
			}
			catch (Exception e){
				e.printStackTrace();

			}
		}
	}

	
	
}
