package com.parking.sourcing;
import java.util.Random;
public class ParkingDataGenerator {

	//observation noise is the noise due to errors in the transportation mode detection, GPS inaccuracy, Driver
	//passenger mis-classifications. In general, the PSE errors are the observation noise

	private final static int MAX_NUM_OF_POINTS_PER_HOUR = 200;  //number of points to be generated per hour

	private int q_i = 0;//historical mean
	private int Q_i = 0; //historical variance or variance of state noise
	private double p_i = 0; // sample ratio
	private double r_i =  0;//mean of the observation noise. 
	private double R_i =  0;// variance of the observation noise
	private Random randomizer;

	private int x_i= 0; // true parking availability

	public  ParkingDataGenerator (){
		q_i = 10;//historical mean
		Q_i = 6; //historical variance or variance of state noise
		p_i = 1; //100% sample ratio
		r_i =  0;//mean of the observation noise. 
		randomizer  = new Random ();
	}


	@SuppressWarnings("unused")
	private int computeTrueParkingAvailability(){


		double state_noise = randomizer.nextGaussian() * Math.sqrt(Q_i); //mean 0 and variance is historical variance Q_i, 
		//the state noise is also known as w_i

		return q_i + (int )state_noise;    //this is x_i

	}

	@SuppressWarnings("unused")
	private int computeObserveParkingAvlbilityByPSD(){

		this.setR_i(0.20*this.getq_i()); // 20% of the historical mean
		double observation_noise = randomizer.nextGaussian()* Math.sqrt(this.getR_i()); //this is observation noise (i.e. v_i)
		//the variance = 20% of true availability

		//with mean r_i and variance R_i
		return  (int) (this.getP_i()*this.getX_i()+observation_noise); //this is a_i the observed parking availability
	}




	public static void main(String args []){
		ParkingDataGenerator PSEGen = new ParkingDataGenerator ();
		int local_x_i = 0; // true parking availability
		int a_i = 0;//observed parking availability
		double total_residualForRMS_1,total_residualForRMS_2,total_residualForRMS_3,total_residualForRMS_4;
		double residual_squared_1, residual_squared_2, residual_squared_3, residual_squared_4;
		double total_RMS_1, total_RMS_2, total_RMS_3,total_RMS_4;
		for (int j=1;j<=3; j++){ //number of hours
			total_residualForRMS_1= total_residualForRMS_2= total_residualForRMS_3= total_residualForRMS_4 = 0;
			if(j==2){
				PSEGen.setq_i(20);// re-setting historical mean
				PSEGen.setQ_i(8);// re-setting historical variance or variance of state noise
			}

			if(j==3){
				PSEGen.setq_i(50);// re-setting historical mean
				PSEGen.setQ_i(20);// re-setting historical variance or variance of state noise
			}

			System.out.println("\nhistorical mean :"+PSEGen.getq_i()+"\nhistorical variance :"+PSEGen.getQ_i());
			System.out.println("\nmean of observation noise :"+PSEGen.getr_i()+"\n");
			System.out.println("\n\n    parking availability" +"-------"+"observe parking availability-------var of observation noise----------state estimation or x hat---------Kalman gain");
			for (int i =1;i<=MAX_NUM_OF_POINTS_PER_HOUR;i++){ //number of data points
				//computing the true parking availability
				local_x_i = PSEGen.computeTrueParkingAvailability();


				//computing the observe parking availability
				PSEGen.setX_i(local_x_i);

				a_i = PSEGen.computeObserveParkingAvlbilityByPSD(); //observe parking availability

				double observation_residual = a_i - PSEGen.getP_i()*PSEGen.getq_i() - PSEGen.getr_i(); //computing  observation_residual

				double Kalman_gain = (PSEGen.getP_i()*PSEGen.getQ_i())/(((PSEGen.getP_i()*PSEGen.getP_i())*PSEGen.getQ_i())+ PSEGen.getR_i()); //computing the Kalman_gain

				double state_estimation = PSEGen.getq_i() + Kalman_gain + observation_residual; //computing state estimation or x hat

				// --------- computing the RMS --------------
				residual_squared_1 = Math.pow((state_estimation - a_i),2);  //RMS (estimate or x hat_1, observed or a_i)
				total_residualForRMS_1 = total_residualForRMS_1 + residual_squared_1;

				residual_squared_2 = Math.pow((PSEGen.getX_i()-state_estimation),2);  //RMS (true avlble or x_i, estimate or x hat_i)
				total_residualForRMS_2 = total_residualForRMS_2 + residual_squared_2;
				
				residual_squared_3 = Math.pow((PSEGen.getX_i()-a_i),2); //RMS (true avlble or x_i, observe or a_i)
				total_residualForRMS_3 = total_residualForRMS_3 + residual_squared_3;
				
				residual_squared_4 = Math.pow((PSEGen.getX_i()-PSEGen.getq_i()),2); //RMS (true avlble or x_i,historical mean)
				total_residualForRMS_4 = total_residualForRMS_4 + residual_squared_4;
				
				//-------------end RMS-----------------------
				System.out.println(i+"\t"+local_x_i+"\t\t\t\t"+ a_i+"\t\t\t\t\t"+PSEGen.getR_i()+"\t\t\t"+(int)state_estimation+"\t\t\t\t"+Kalman_gain);

			}

			total_RMS_1 = Math.sqrt((total_residualForRMS_1/MAX_NUM_OF_POINTS_PER_HOUR));
			System.out.println("ROOT MEAN SQUARE (estimate , observed )= "+total_RMS_1 ); //we should vary this with the penetration ratio .....
			//....when the penetration ratio is high the error is small. As the penetration ratio decreases, the size of error increases since we'll
			// have less users of the PSE we'll have less observed activities
			
			
			total_RMS_2 = Math.sqrt((total_residualForRMS_2/MAX_NUM_OF_POINTS_PER_HOUR));
			System.out.println("ROOT MEAN SQUARE (true avlble, estimate )= "+total_RMS_2 );
			
			
			total_RMS_3 = Math.sqrt((total_residualForRMS_3/MAX_NUM_OF_POINTS_PER_HOUR));
			System.out.println("ROOT MEAN SQUARE (true avlble , observed )= "+total_RMS_3);
			
			
			total_RMS_4 = Math.sqrt((total_residualForRMS_4/MAX_NUM_OF_POINTS_PER_HOUR));
			System.out.println("ROOT MEAN SQUARE (true avlble,historical mean )= "+total_RMS_4 );



		}

	}


	public int getQ_i() {
		return Q_i;
	}

	public void setq_i(int q_i) {
		this.q_i = q_i;
	}

	public int getq_i() {
		return q_i;
	}

	public void setQ_i(int q_i) {
		this.Q_i = q_i;
	}

	public double getP_i() {
		return p_i;
	}

	public void setP_i(double p_i) {
		this.p_i = p_i;
	}

	public double getr_i() {
		return r_i;
	}

	public void setr_i(double r_i) {
		this.r_i = r_i;
	}

	public double getR_i() {
		return R_i;
	}

	public void setR_i(double r_i) {
		R_i = r_i;
	}

	public int getX_i() {
		return x_i;
	}

	public void setX_i(int x_i) {
		this.x_i = x_i;
	}





}
